import { Constructor } from '../utils'
import { serializeWith } from './impl'
import { isGenericSerializer, Serializer } from './interface'

export function serialize<T extends Constructor>(constructor: T) {
  class Serializable extends constructor implements Serializable {
    static name = constructor.name
    serialize<U>(serializer: Serializer<U>): U {
      // shortcut for serializers with only the serializeAny method
      if (isGenericSerializer(serializer)) {
        return serializer.serializeAny!(this) as U
      } else {
        return serializeWith(serializer, this) as U
      }
    }
  }

  return Serializable
}
